;**************************** Differential Thermostat ******************
;
;   File name   :  difterm.asm
;   AUTHOR      :  Valentin Kulikov, VKU2007
;   E-mail		:  kulikov@elf.stuba.sk
;   VERSION     :  1.02
;   DATE        :  20.FEB.07
;   Internal osc:  1.6 MHz
;   MCU			:  ATtiny 15L
;   Compiler    :  Atmel AVR Assembler
;   Contents    :  
;
;   Function	:  Diferential thermostat for heting system cyrculating pump
;
;   Additional	:  Note: EEPROM not used
;   FUSES		:  RSTDISBL, CKSEL1, CKSEL0, SPIEN, BODLEVEL, BODEN  
;	Note		:  Sensors LM35D, voltage output 10 mV / C, tol. 1C
;				   0C ~ 0,6V, 100C ~ 1,6V
;			:Fixed Watch DOG settings 9.April 2007 (v1.02)	
;
;************************************************************************

; Define files 
.include	"tn15def.inc"

.def	Temp	=r16	;temporary register
.def	Temp1	=r17	;temporary register #1
.def	dly		=r18	;delay register
.def	dlyl	=r19	;long delay
.def	dlyl1	=r20	;long delay 1

;************************* IN/OUT pin definition ************************
;Differential GAIN AMP PB3 (+) and PB4 (-)

.equ	Pout	= 2		;PB2 - remote signal for Pump (Active=Lo)
.equ	Heatin	= 5		;PB1 - detect room thermostat status (PB5/RST usable as input)
						;PB5/RST was not possible to configure as output with open drain!
.equ	Heatout	= 0		;PB0 - if PB5 low then PB4 high (heating Active=Lo) 0
.equ	Err_sig	= 1		;PB5 - Sensor Error signalization, Active in low (open Drain)

						; PB0 & PB1 Analog comparator

.equ	hold	= 8		;hold time, time period when pump is default on
						;~1 minute @ hold=256 

;******************************* Constants ******************************

.equ	Vx_min	=0x10		;limit = 0,3V, Ref=5V, ADC 8bit (0.3*255/5=0x10)		
.equ	Vx_max	=0x66		;limit = 2V, Ref=5V, ADC 8bit (2*255/5=0x66)
.equ	Vx_ndiff=0x0D		;negative difference limit 25C (25*10mV*255/5=0x0D)
.equ	TREF5C	=0x33		;temperature reference 5C (5*10mV*20*255/5); GAIN=20
.equ	TREF3C	=0x1F		;temperature reference 3C (3*10mV*20*255/5)

;******************************** MACROS ********************************
.MACRO Ltimerrst               ;start macro Long timer reset 
	clr		dlyl			;reset long delay counter	
	clr		dlyl1			;reset long delay 1 counter
.ENDMACRO                   ; End macro definition

.MACRO dly60s				;start macro timer 1 (1 minute) 
	ldi	dly, 0xC8			;delay ~20 sec
	rcall	DELAY
	ldi	dly, 0xC8			;delay ~20 sec
	rcall	DELAY
	ldi	dly, 0xC8			;delay ~20 sec
	rcall	DELAY

.ENDMACRO                   ; End macro definition 

.MACRO dly20s				;start macro timer 1 (1 minute) 
	ldi	dly, 0xC8			;delay ~20 sec
	rcall	DELAY
.ENDMACRO                   ; End macro definition 
 
;************************** Interrupt Vector Table **********************
; 1 Byte each:

.ORG $0000

		rjmp 	RST 		; Reset handler
		reti				; IRQ0 handler
		reti				; Pin change handler
		reti				; Timer0 overflow handler
		reti				; EEPROM Ready handler
		reti				; Analog Comparator handler
 
		.db	"Diferential thermostat   ",0
		.db	"Ver. 1.0 by VKU2007",0 

RST:	
		clr	temp
		out	PORTB, temp			;reset PORTB
		out	DDRB, temp			;set all pins as inputs

		sbi	DDRB, Pout			;set pin as output
		sbi	PORTB, Pout			;set to high (pump off)
	
		sbi	DDRB, Heatout		;set pin as output
		cbi	PORTB, Heatout		;set to low (heating off)

		sbi	DDRB, Err_sig		;set pin as output
		sbi	PORTB, Err_sig		;set Error signal to high

		ldi	temp, 0x0F			;Set Watchdog prescaler to 2048
		out	WDTCR, temp			;FIXED 9.April 2007	

		ldi	temp, 0x00			;set EEPROM address
		out	EEAR, temp			;adress 0x00 = inter.osc.cal.

		ldi	temp, 0x01			;strobe from EEPROM to EEDR
		out	EECR, temp		

		in	temp, EEDR			;read internal oscil. cal. value

		out OSCCAL, temp		;write factory callibration byte to FLASH
		Ltimerrst				;reset Long delay (macro)

;********Run Pump for 20 sec at START***************************
		
		cbi		PORTB, Pout		;set Pump ON, Refresh and system check
		dly20s
		sbi		PORTB, Pout		;set Pump OFF

;********Check Heatin*******************************************

MAIN:

		sbic 	PINB, Heatin	;if Heating low then go to HEAT 
		rjmp	HEATOFF

		cbi		PORTB, Pout		;set Pump ON
		sbi		PORTB, Heatout	;set to low, heating off
		
		ldi	dly, 0x05			;delay ~0.5 sec
		rcall	DELAY
		
		rjmp	MAIN			;jump to MAIN (cycle till Heatin Lo)

HEATOFF:
		cbi		PORTB, Heatout	;set to low, heating off
		
;********Check t1 Sensor****************************************

								;ADC2 configuration & Vdif measurement
		ldi		temp, 0x22		;5 V Vcc as Ref, NO Bypas CAP on AREF A7
		out		ADMUX, temp		;MUX=ADC2
								;Reolution 8bit, 
								;ADLAR set - Result adjusted Left
		ldi		temp, 0x85		;Bit7: ADEN: ADC Enable
		out		ADCSR, temp		;Bit6: ADSC: ADC Start Conversion
								;Bit5: ADFR=0 Free runing Disabled
								;Bit4: ADIF: ADC Intesrrupt Flag == 1 when ADC compeeted
								;Bit3: ADIE: ADC Interrupt Enable
								;Bits2-0: ADPS2..ADPS0: ADC Prescaler Select Bits
								;Prescaler=32 (FADC~50kHz)

;		ldi		dly, 0x01		;delay(100ms) ~ 100 ms
;		rcall	DELAY
		nop
		sbi		ADCSR, 6		;start single ADC conversion
		rcall	WAITADC			;wait till ADC compleeted

		in		temp, ADCH		;get 8bit ADC value 
		cpi		temp, Vx_min	;compare with Vx_min
		brlo	T1FAIL			;branch to T2PASS if ADCH >= 0,5V

		cpi		temp, Vx_max	;compare 3V, Ref=5V, ADC 8bit (3*255/5=0x99)
		brlo	T1PASS			;branch to T2PASS if ADCH < 3V

T1FAIL:
		sbi		PORTB, Pout		;set Pump OFF
		cbi		PORTB, Err_sig	;Error signal to low (signal ON)
		rjmp	MAIN			;jump to MAIN, sensor t1 connection fail

T1PASS:	
		mov temp1,temp
		ldi	temp, Vx_ndiff		;negative difference T1>T2!
		add	temp1,temp 

;********Check t2 Sensor & order check***************************

								;ADC3 configuration & Vdif measurement
		ldi		temp, 0x23		;5 V Vcc as Ref, NO Bypas CAP on AREF A7
		out		ADMUX, temp		;MUX=ADC3
								;Reolution 8bit, 
								;ADLAR set - Result adjusted Left
		ldi		temp, 0x85		;Bit7: ADEN: ADC Enable
		out		ADCSR, temp		;Bit6: ADSC: ADC Start Conversion
								;Bit5: ADFR=0 Free runing Disabled
								;Bit4: ADIF: ADC Intesrrupt Flag == 1 when ADC compeeted
								;Bit3: ADIE: ADC Interrupt Enable
								;Bits2-0: ADPS2..ADPS0: ADC Prescaler Select Bits
								;Prescaler=32 (FADC~50kHz)
		nop
		sbi		ADCSR, 6		;start single ADC conversion
		rcall	WAITADC			;wait till ADC compleeted

		in		temp, ADCH		;get 8bit ADC value 			
		
		cp		temp1, temp		;temp1, measuremtn from ADC2 +infput of GAIN AMP
		brlo	T2FAIL			;check if sensors are in correct order for Tdiff>10C

		cpi		temp, Vx_min	;compare with Vx_min
		brlo	T2FAIL			;branch to T2PASS if ADCH >= Vx_min

		cpi		temp, Vx_max	;compare with Vx_max
		brlo	T2PASS			;branch to T2PASS if ADCH < Vx_max	

T2FAIL:
		sbi		PORTB, Pout		;set Pump OFF
		cbi		PORTB, Err_sig	;Error signal to low (Signal ON)
		rjmp	MAIN			;jump to MAIN, sensor t1 connection fail

T2PASS:
		sbi		PORTB, Err_sig	;Error signal to low (Err Signal OFF)
		
;********Check tenperature difference ***************************

								;ADC configuration & Vdif measurement
		ldi		temp, 0x27		;Vcc (5V) as REF, NO Bypas CAP on AREF
		out		ADMUX, temp		;ADC2-positive, ADC3-negative bias, GAIN=20
								;Reolution 8bit, ADLAR set - Result adjusted Left
		ldi		temp, 0x85		;Bit7: ADEN: ADC Enable
		out		ADCSR, temp		;Bit6: ADSC: ADC Start Conversion
								;Bit5: ADFR=0 Free runing Disabled
								;Bit4: ADIF: ADC Intesrrupt Flag	== 1 when ADC compeeted
								;Bit3: ADIE: ADC Interrupt Enable
								;Bits2-0: ADPS2..ADPS0: ADC Prescaler Select Bits
								;Prescaler=32 (FADC~50kHz)
		nop
		sbi		ADCSR, 6		;start single ADC conversion
		rcall	WAITADC			;wait till ADC compleeted
	
		ldi		temp, 0x01		;set EEPROM address
		out		EEAR, temp		;adress 0x01 = TREF value 3C to 15C
		ldi		temp, 0x01		;strobe from EEPROM to EEDR
		out		EECR, temp		
		in		temp1, EEDR		;read internal oscil. cal. value

		cpi		temp1, TREF3C	;check if TREF > 3C
		brsh	TREF_EEPROM		
		ldi		temp1, TREF5C	;if TREF in EEPROM < 3C, then use 5C as default
		
TREF_EEPROM:

		cpi		temp1, 0xFF		;check if EEPROM unprogrammed
		brlo	TREF_EEPROM1
		ldi		temp1, TREF5C	;if TREF in EEPROM is 0xFF, then use 5C as default

TREF_EEPROM1:
		

		in		temp, ADCH		;get 8bit ADC value, Gain 20
		cp		temp, temp1		;compare temp. difference with TREF
								
		brlo	CONT1			;branch to CONT1 if diff less then spec.
		cbi		PORTB, Pout		;set Pump ON
		Ltimerrst				;reset Long Timer

		ldi		dly, 0x0B		;10 x delay(100ms) ~ 1 s
		rcall	DELAY
	
		rjmp	MAIN

CONT1:
		
		sbic	PORTB,	Pout		;Skip if PUMP previously ON and leave Pump
		rjmp	CONT2				;runing for 60 sec
		dly60s

CONT2:			
		sbi		PORTB,	Pout		;set Pump OFF

		ldi		dly,	0x14		;delay(100ms) ~ 2 s
		rcall	DELAY				;sum 0.1sec (ADC)+0.9sec ~ 2sec
	
;**************** 24h timer, cycle ~ 2sec eachs****************************

		cpi 	dlyl,0xB4 		;compare, start from 0x00 / one cycle ~2 sec
		inc		dlyl			;increment dlyl, time ~ 6 minutes (180x2sec)
		brlo	END				;branch to MAIN if lower
		clr		dlyl			;reset long delay counter #0

		cpi 	dlyl1,0xF0		;time = 2sec * (dlyl)*dlyl1
		inc		dlyl1			;increment dlyl1 (6 min x 240 ~ 24h) 
		brlo	END				;if lower then jump to MAIN, else continue

;***************Pump Refreshing, run for ~60 sec****************************
REFRESH:
		cbi		PORTB, Pout		;set to high, pump ON
		dly60s					;macro timer 1
		Ltimerrst				;reset Long delay (macro)
		
		sbic 	PINB, Heatin	;if Heating occured during 60sec delay
								;Pump will stay ON
		sbi		PORTB, Pout		;set to high, pump OFF		

END:	rjmp	MAIN
;***************Subroutines*************************************************
								;if long timer reached then switch pump
WAITADC:
		sbis	ADCSR, 4		;Wait till ADC completetd
		rjmp 	WAITADC
		ret
							
	
DELAY:							;dly * 0.1 ms (100 ms -> dly=0x01)			
		ldi 	temp,0x00		
		out 	TIMSK,temp		;Disable timer interupt

		ldi 	temp,0x05		;Start triger 0 with prescaller
		out 	TCCR0,temp		;set prescaler = CLK/1024 ~0.64 ms @ 1.6 MHz
		
		ldi 	temp,0x64		;counter start value (up-couning) 8bit
		out 	TCNT0,temp		;delay (0,099840 sec) = (255-100)*1024/1600000
loopt:	in   	temp,TIFR
 		wdr						;watch dog reset
		sbrs	temp,1			;delay time reached? (overflow limit)
 		rjmp 	loopt

		ldi 	temp,0x02		
		out 	TIFR,temp		;clear int T1 flag (one posibility)
								;overwriting by log1 reuired 
								;for clearing TIFR flag
		ldi 	temp,0x00		;Disable triger 
		out 	TCCR0,temp	
		
				;checkin Heatin during long delay
		;in	 	temp,PINB	;check Heating low then go to MAIN
		sbis	PINB, Heatin
		rjmp	MAIN	

				dec		dly			;loop dly*=0,099840 seconds
				cpi		dly, 0x00
				brne 	DELAY		;max 25,56 sec
		ret


;That all folks
;VKU2007
